

/***************************************************************************
 *        Main.c
 *
 *  Copyright  2006  Jonas Broms <sanoix@gmail.com>
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <unistd.h>

#include <SDL.h>

#define TICK_INTERVAL	40
#define TURN		0.05
#define HOLE_SIZE	4

#define MAX_PLAYERS	8

unsigned char keystate[1024];
Uint8 players = 0;
Uint32 c_black, c_white;

typedef struct {
	double head_x, head_y, direction;
	Uint8 is_alive, is_in, score, hole_counter;
	Uint32 color;
	SDLKey left, right;
} Worm;

Worm worms[MAX_PLAYERS];

int Draw_Text(SDL_Surface *screen, char *str, Uint32 color, Sint16 x, Sint16 y, Uint8 size);
int Text_Init();

/*
 * FUNCTIONS
 */

Uint32 Get_Pixel(SDL_Surface *screen, Sint32 x, Sint32 y)
{
        Uint8 *bufp;
        bufp = (Uint8 *)screen->pixels + y*screen->pitch + x;
	return *bufp;
}

void Draw_Score(SDL_Surface *screen)
{
	int i;
	SDL_Rect r;
	char buf[16];

	r.x = 762; r.y = 6; r.w = 32; r.h = 588;
	SDL_FillRect(screen, &r, c_black);
	SDL_UpdateRect(screen, r.x, r.y, r.w, r.h);

	for (i=0; i < MAX_PLAYERS; i++) {
		if (!worms[i].is_in)
			continue;

		snprintf(buf, sizeof(buf) - 1, "%d", worms[i].score);
		Draw_Text(screen, buf, worms[i].color, 765, 30 + (20 * i), 0);
	}
}

int Move_Worms(SDL_Surface *screen)
{
	int i, j;
	double x_drift, y_drift;

	for (i=0; i < MAX_PLAYERS; i++) {
		if (!worms[i].is_in || !worms[i].is_alive)
			continue;

		if (keystate[worms[i].right])
			worms[i].direction -= TURN;

		if (keystate[worms[i].left])
			worms[i].direction += TURN;

		x_drift = 1.5 * sin(worms[i].direction);
		y_drift = 1.5 * cos(worms[i].direction);

		if (Get_Pixel(screen, (Sint16)(worms[i].head_x+(2.25*x_drift)),
					(Sint16)worms[i].head_y+(2.25*y_drift)) != c_black) {
			worms[i].is_alive = 0;
			players--;

			for(j=0; j<MAX_PLAYERS; j++)
				if (worms[j].is_alive && worms[j].is_in) worms[j].score++;

			Draw_Score(screen);
		} 

		worms[i].head_x += x_drift;
		worms[i].head_y += y_drift;

	}

	return 0;
}

void Draw_Worms(SDL_Surface *screen)
{
	int i;
	SDL_Rect r;

	for (i=0; i < MAX_PLAYERS; i++) {
		if (!worms[i].is_in || !worms[i].is_alive)
			continue;

		r.x = (Sint16)(worms[i].head_x - 1.5);
		r.y = (Sint16)(worms[i].head_y - 1.5);
		r.w = r.h = 3;
		
		worms[i].hole_counter--;
		
		if (worms[i].hole_counter <= 0) {
			worms[i].hole_counter = HOLE_SIZE + 100 + (rand() % 100);
		} else if (worms[i].hole_counter < HOLE_SIZE) {
			SDL_FillRect(screen, &r, c_black);
		} else {
			SDL_FillRect(screen, &r, worms[i].color); 
		}
	}

}

Uint32 TimeLeft(int interval)
{
	static Uint32 next_time = 0;
	Uint32 now;

	now = SDL_GetTicks();
	if ( next_time <= now ) {
		next_time = now+interval;
		return(0);
	}
	return(next_time-now);
}

void Erase_Ok(SDL_Surface *screen, Sint16 x, Sint16 y)
{
	SDL_Rect r;
	r.h = 20;
	r.w = 40;
	r.x = x;
	r.y = y;
	SDL_FillRect(screen, &r, c_black); 
	SDL_UpdateRect(screen, r.x, r.y, r.w, r.h);
}


void Draw_Interval(SDL_Surface *screen, Sint16 x, Sint16 y, int interval)
{
	char buf[16];

	Erase_Ok(screen, x, y);
	snprintf(buf, sizeof(buf) - 1, "%d", interval);
	Draw_Text(screen, buf, c_white, x, y, 0);
}

void Init_Worms(void)
{
	int i;

	for(i=0; i<MAX_PLAYERS; i++) {
		worms[i].head_x = 100.0 + (rand() % 600);
		worms[i].head_y = 50.0 + (rand() % 500);
		worms[i].direction = (double)rand() / (double)rand();
		worms[i].hole_counter = HOLE_SIZE + 100 + (rand() % 100);
		worms[i].is_alive = 1;
	}

}

void Show_Splash(SDL_Surface *screen, int *interval)
{
	int i;

	SDL_Event event;

	SDL_FillRect(screen, NULL, c_black);
	SDL_UpdateRect(screen, 0, 0, 0, 0);

	Draw_Text(screen, "Battle Worms v1.2", c_white, 100, 50, 1);

	Draw_Text(screen,"Game interval:", c_white, 100, 200, 0);
	Draw_Interval(screen, 350, 200, *interval);

	Draw_Text(screen, "(left / right)", worms[0].color, 100, 300, 0);
	Draw_Text(screen, "(z / x)", worms[1].color, 100, 320, 0);
	Draw_Text(screen, "(m / ,)", worms[2].color, 100, 340, 0);
	Draw_Text(screen, "(0 / +)", worms[3].color, 100, 360, 0);
	Draw_Text(screen, "(kp1 / kp2)", worms[4].color, 100, 380, 0);
	Draw_Text(screen, "(kp8 / kp9)", worms[5].color, 100, 400, 0);
	Draw_Text(screen, "(f / g)", worms[6].color, 100, 420, 0);
	Draw_Text(screen, "(page up / page down)", worms[7].color, 100, 440, 0);

	Draw_Text(screen, "coded by Jonas <sanoix@gmail.com>", c_white, 450, 550, 0);

	for (i=0; i < MAX_PLAYERS; i++)
		worms[i].is_in = worms[i].is_alive = worms[i].score = 0;

	while (1) {
		SDL_WaitEvent(&event);
		if (event.type == SDL_KEYDOWN) {
			switch (event.key.keysym.sym) {
			case SDLK_UP:
				*interval = (*interval >= 300) ? 300 : *interval + 1;
				Draw_Interval(screen, 350, 200, *interval);	
				break;
			case SDLK_DOWN:
				*interval = (*interval <= 1) ? 1 : *interval - 1;
				Draw_Interval(screen, 350, 200, *interval);	
				break;
			case SDLK_LEFT:
				Draw_Text(screen, "OK", worms[0].color, 350, 300, 0);
				worms[0].is_in = worms[0].is_alive = 1;
				break;
			case SDLK_RIGHT:
				Erase_Ok(screen, 350, 300);
				worms[0].is_in = worms[0].is_alive = 0;
				break;
			case SDLK_z:
				Draw_Text(screen, "OK", worms[1].color, 350, 320, 0);
				worms[1].is_in = worms[1].is_alive = 1;
				break;
			case SDLK_x:
				Erase_Ok(screen, 350, 320);
				worms[1].is_in = worms[1].is_alive = 0;
				break;
			case SDLK_m:
				Draw_Text(screen, "OK", worms[2].color, 350, 340, 0);
				worms[2].is_in = worms[2].is_alive = 1;
				break;
			case SDLK_COMMA:
				Erase_Ok(screen, 350, 340);
				worms[2].is_in = worms[2].is_alive = 0;
				break;
			case SDLK_0:
				Draw_Text(screen, "OK", worms[3].color, 350, 360, 0);
				worms[3].is_in = worms[3].is_alive = 1;
				break;
			case SDLK_PLUS:
				Erase_Ok(screen, 350, 360);
				worms[3].is_in = worms[3].is_alive = 0;
				break;
			case SDLK_KP1:
				Draw_Text(screen, "OK", worms[4].color, 350, 380, 0);
				worms[4].is_in = worms[4].is_alive = 1;
				break;
			case SDLK_KP2:
				Erase_Ok(screen, 350, 380);
				worms[4].is_in = worms[4].is_alive = 0;
				break;
			case SDLK_KP8:
				Draw_Text(screen, "OK", worms[5].color, 350, 400, 0);
				worms[5].is_in = worms[5].is_alive = 1;
				break;
			case SDLK_KP9:
				Erase_Ok(screen, 350, 400);
				worms[5].is_in = worms[5].is_alive = 0;
				break;
			case SDLK_f:
				Draw_Text(screen, "OK", worms[6].color, 350, 420, 0);
				worms[6].is_in = worms[6].is_alive = 1;
				break;
			case SDLK_g:
				Erase_Ok(screen, 350, 420);
				worms[6].is_in = worms[6].is_alive = 0;
				break;
			case SDLK_PAGEUP:
				Draw_Text(screen, "OK", worms[7].color, 350, 440, 0);
				worms[7].is_in = worms[7].is_alive = 1;
				break;
			case SDLK_PAGEDOWN:
				Erase_Ok(screen, 350, 440);
				worms[7].is_in = worms[7].is_alive = 0;
				break;
			case SDLK_SPACE:
				for(i=0, players=0; i < MAX_PLAYERS; i++) {
					if (worms[i].is_in)
						players++;
				}
				if (players > 1) return;
				break;
			case SDLK_ESCAPE:
			case SDL_QUIT:
				exit(0);
				break;
			default:
				break;
			}
		}
	}
}

void Restart_Game(SDL_Surface *screen)
{
	int i;
	SDL_Rect r;

	Init_Worms();

	SDL_FillRect(screen, NULL, c_black);

	r.x=0;r.y=0;r.w=800;r.h=5;
	SDL_FillRect(screen, &r, c_white);

	r.x=0;r.y=0;r.w=5;r.h=600;
	SDL_FillRect(screen, &r, c_white);

	r.x=760;r.y=0;r.w=2;r.h=600;
	SDL_FillRect(screen, &r, c_white);

	r.x=795;r.y=0;r.w=5;r.h=600;
	SDL_FillRect(screen, &r, c_white);

	r.x=0;r.y=595;r.w=800;r.h=5;
	SDL_FillRect(screen, &r, c_white);

	SDL_UpdateRect(screen, 0, 0, 0, 0);

	Draw_Score(screen);

	for (i=0; i < MAX_PLAYERS; i++) {
		int j;
		if (!worms[i].is_in)
			continue;

		r.x = (Sint16)(worms[i].head_x - 1.5);
		r.y = (Sint16)(worms[i].head_y - 1.5);
		r.w = r.h = 3;

		for (j=0; j<6; j++) {
			SDL_FillRect(screen, &r, c_black); 
			SDL_UpdateRect(screen, r.x, r.y, r.w, r.h);
			SDL_Delay(60);
			SDL_FillRect(screen, &r, worms[i].color); 
			SDL_UpdateRect(screen, r.x, r.y, r.w, r.h);
			SDL_Delay(60);
		}
	}
	
}

int main( int argc, char *argv[] )
{
	SDL_Surface *screen;
	SDL_Event event;
	int interval = TICK_INTERVAL;

	if (argc == 2)
		interval = atoi(argv[1]);

	screen = SDL_SetVideoMode(800, 600, 8, SDL_HWSURFACE | SDL_FULLSCREEN);
	if ( screen == NULL ) {
		printf("Unable to set 800x600 video: %s\n", SDL_GetError());
		exit(1);
	}

	atexit(SDL_Quit);
	
	SDL_ShowCursor(SDL_DISABLE);

	if (Text_Init() != 0)
		exit(1);

	/* Seed the random number generator */
	srand(time(NULL));

	c_black = SDL_MapRGB(screen->format, 0, 0, 0);
	c_white = SDL_MapRGB(screen->format, 255, 255, 255);

	worms[0].color = SDL_MapRGB(screen->format, 255, 0, 0);
	worms[1].color = SDL_MapRGB(screen->format, 255, 255, 0);
	worms[2].color = SDL_MapRGB(screen->format, 0, 255, 255);
	worms[3].color = SDL_MapRGB(screen->format, 0, 0, 255);
	worms[4].color = SDL_MapRGB(screen->format, 255, 0, 255);
	worms[5].color = SDL_MapRGB(screen->format, 0, 255, 0);
	worms[6].color = SDL_MapRGB(screen->format, 100, 100, 0);
	worms[7].color = SDL_MapRGB(screen->format, 255, 0, 100);

	worms[0].left = SDLK_LEFT;
	worms[0].right = SDLK_RIGHT;
	worms[1].left = SDLK_z;
	worms[1].right = SDLK_x;
	worms[2].left = SDLK_m;
	worms[2].right = SDLK_COMMA;
	worms[3].left = SDLK_0;
	worms[3].right = SDLK_PLUS;
	worms[4].left = SDLK_KP1;
	worms[4].right = SDLK_KP2;
	worms[5].left = SDLK_KP8;
	worms[5].right = SDLK_KP9;
	worms[6].left = SDLK_f;
	worms[6].right = SDLK_g;
	worms[7].left = SDLK_PAGEUP;
	worms[7].right = SDLK_PAGEDOWN;

	Show_Splash(screen, &interval);
	Restart_Game(screen);

	while (1) {
		if (SDL_PollEvent(&event)) {
			switch (event.type) {
			case SDL_KEYDOWN:
				if (event.key.keysym.sym == SDLK_ESCAPE) {
					Show_Splash(screen, &interval);
					Restart_Game(screen);
				} if (event.key.keysym.sym == SDLK_SPACE && players <= 1) {
					int i;
					for(i=0, players=0; i < MAX_PLAYERS; i++) {
						if (worms[i].is_in)
							players++;
					}
					
					for (i=0; i < MAX_PLAYERS; i++) {
						if (worms[i].score >=players*5) {
							Draw_Text(screen, "Game Over!!", c_white, 230, 250, 1);
							SDL_Delay(4000);
							Show_Splash(screen, &interval);
							break;
						}
					}
					
					Restart_Game(screen);
				}
				else
					keystate[event.key.keysym.sym] = 1;
				break;
			case SDL_KEYUP:
				keystate[event.key.keysym.sym] = 0;
				break;
			case SDL_QUIT:
				exit(0);
				break;
			default:
				break;
			}
		}

		if (players > 1) {
			Move_Worms(screen);
			Draw_Worms(screen);
			SDL_UpdateRect(screen, 0, 0, 0, 0);
		}

		SDL_Delay(TimeLeft(interval));
	}

	return 0;
}


