/***************************************************************************
 *        Text.c
 *
 *  Copyright  2006  Jonas Broms <sanoix@gmail.com>
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <SDL.h>
#include <SDL_ttf.h>

#define FONT_PATH        "/usr/share/games/battleworms/font.ttf"


TTF_Font *font_small = NULL, *font_big = NULL;

int Draw_Text(SDL_Surface *screen, char *str, Uint32 color, Sint16 x, Sint16 y, Uint8 size)
{
	SDL_Surface *text;
	SDL_Rect dstrect;
	SDL_Color c;
	TTF_Font *font;

	font = (size) ? font_big : font_small;

	SDL_GetRGB(color, screen->format, &c.r, &c.g, &c.b);

	if (!(text=TTF_RenderText_Blended(font, str, c))) {
        	printf("Error: %s\n", TTF_GetError());
		return -1;
	}

	if ( text != NULL ) {
		dstrect.x = x;
		dstrect.y = y;
		dstrect.w = text->w;
		dstrect.h = text->h;
		SDL_BlitSurface(text, NULL, screen, &dstrect);
		SDL_UpdateRect(screen, x, y, text->w, text->h);
		SDL_FreeSurface(text);
	}

	return 0;
}

int Text_Init()
{
	char font[512];

	if (access(FONT_PATH, R_OK) == 0) {
		strncpy(font, FONT_PATH, sizeof(font) - 1);
	} else if (access("./font.ttf", R_OK) == 0) {
		strncpy(font, "./font.ttf", sizeof(font) - 1);
	} else {
		printf("No font.ttf file found!\n");
		return -1;
	}


	if (TTF_Init() == -1) {
		fprintf(stderr, "TTF_Init: %s\n", TTF_GetError());
		return -1;
	}

	atexit(TTF_Quit);

	font_small=TTF_OpenFont(font, 16);
	if (!font_small) {
		fprintf(stderr, "TTF_OpenFont: %s\n", TTF_GetError());
		return -1;
	}

	font_big=TTF_OpenFont(font, 48);
	if (!font_big) {
		fprintf(stderr, "TTF_OpenFont: %s\n", TTF_GetError());
		return -1;
	}

	return 0;
}
